require 'sdl'

require 'interface.rb'
require 'enemy.rb'

# ARE components
require 'areanimation.rb'

TARGET_FPS = 30
MAX_ENEMIES = 30
PERIOD_TIME = 1000 * 10 # 10 seconds

SDL.init(SDL::INIT_VIDEO)
SDL::TTF.init
SDL::WM.setCaption 'Defender of Mars', 'Defender of Mars'

font = SDL::TTF.open('./gfx/font.ttf', 16)
font.style = SDL::TTF::STYLE_NORMAL
fontsmall = SDL::TTF.open('./gfx/font.ttf', 12)
fontsmall.style = SDL::TTF::STYLE_NORMAL
screen = SDL::setVideoMode(720, 480, 16, SDL::HWSURFACE + SDL::DOUBLEBUF)
interface = Interface.new(screen)

menu_sprite = ARESprite.new(screen, true)
menu_sprite.sprLoad('./gfx/menu.bmp', 720, 1, 0, false)
game_over = ARESprite.new(screen, true)
game_over.sprLoad('./gfx/gameover.bmp', 720, 1, 0, false)

frame_counter = 1
last_check_time = 0
current_fps = 0
wave = 0
aliens = Array.new
anims = Array.new
running = true
last_period = 0
# 0 = menu, 1 = game, 2 = game over
game_state = 0

while running
  elapsed = SDL.getTicks - last_check_time;
  if elapsed > 1000 then
    current_fps = frame_counter;
    last_check_time = SDL.getTicks
    frame_counter = 0
  end
  screen.fillRect(0, 0, 720, 480, 0)
  
  if game_state == 0
    menu_sprite.render(screen, 0, 0)
    
    while (event = SDL::Event2.poll) do
      case event
      when SDL::Event2::Quit
        running = false
      when SDL::Event2::KeyUp
        game_state = 1 if event.sym == SDL::Key::SPACE
      end
    end
  end
  
  if game_state == 2
    game_over.render(screen, 0, 0)
    
    while (event = SDL::Event2.poll) do
      case event
      when SDL::Event2::Quit
        running = false
      when SDL::Event2::KeyUp
        game_state = 1 if event.sym == SDL::Key::SPACE
        
        frame_counter = 1
        last_check_time = 0
        current_fps = 0
        wave = 0
        aliens = Array.new
        anims = Array.new
        last_period = 0
        interface = Interface.new(screen)
      end
    end
  end
  
  if game_state == 1
    if SDL.getTicks - last_period >= PERIOD_TIME then
      # process events in new period
      interface.mars_year += 1
      # add bonuses
      MAP_S.times do |i|
        MAP_S.times do |j|
          if not interface.mars_map[i][j].nil? then
            b = interface.mars_map[i][j].bonuses
            interface.mars_credits += b[0]
            interface.mars_research += b[1]
            interface.mars_people += b[2]
          end
        end
      end

      # generate enemies
      if (rand(6) + 1 < 5) then
        count = (5 + wave * rand(2))
        count = MAX_ENEMIES if count > MAX_ENEMIES
        count.times do |i|
          mass = 300 + rand(1000)
          vel = AREVector.new(rand(3), rand(1))
          hp = 10 + wave * rand(20)
          lasers = rand(3) + 1
          damage = 1 + rand(8)
          alien = Enemy.new(screen, true)
          alien.sprLoad('./gfx/alien' + rand(ENEMIES_COUNT).to_s + '.bmp', 16, 1, 0, true)
          alien.setEnemy(mass, vel, hp, lasers, damage)
          alien.position = AREVector.new(50 + rand(150), 50 + rand(380))
          aliens << alien

          # create warp wave
          ani = AREAnimationVec.new(screen, true)
          ani.sprLoad('./gfx/warpwave.bmp', 32, 30, 0, true)
          ani.setAnimation(0, 5, false)
          ani.position = AREVector.new(alien.position.x - 5, alien.position.y - 5)
          ani.start
          anims << ani
        end
        wave += 1
        interface.mars_message = PIRATES_MSGS[rand(PIRATES_MSGS.size)]
      end
      last_period = SDL.getTicks
    end

    while (event = SDL::Event2.poll) do
      case event
      when SDL::Event2::Quit
        running = false
      when SDL::Event2::MouseButtonUp
        interface.clicked
      when SDL::Event2::MouseMotion
        interface.mousemove
      end
    end

    interface.drawFPS(current_fps, font)
    interface.draw(font, fontsmall)

    # process aliens
    aliens.each do |alien|
      alien.render(screen)
      alien.move
      alien.fire(interface.mars_map, screen)
      # check if alien should "bomb the mars!"
      if Math.sqrt((MARS_CENTER - alien.position).sumsq) < 40 + rand(80) and alien.havebomb
        ani = AREAnimationVec.new(screen, true)
        ani.sprLoad('./gfx/nboom.bmp', 36, 27, 0, true)
        ani.setAnimation(0, 10, false)
        ani.position = alien.position
        ani.start
        anims << ani
        interface.mars_hp -= rand(50) if interface.mars_hp > 0
        game_state = 2 if interface.mars_hp < 0
        alien.havebomb = false
      end
      if alien.out and alien.hp <= 0 then
        ani = AREAnimationVec.new(screen, true)
        ani.sprLoad('./gfx/boom.bmp', 32, 19, 0, true)
        ani.setAnimation(0, 10, false)
        ani.position = alien.position
        ani.start
        anims << ani
      end
      aliens.delete(alien) if alien.out
    end
    # process buildings
    MAP_S.times do |i|
      MAP_S.times do |j|
        if not interface.mars_map[i][j].nil? then
          bld = interface.mars_map[i][j]
          bld.fire(aliens, screen) if bld.army
          if bld.destroyed
            ani = AREAnimationVec.new(screen, true)
            ani.sprLoad('./gfx/nboom.bmp', 36, 27, 0, true)
            ani.setAnimation(0, 10, false)
            ani.position = AREVector.new(460 + i * TILE_W, 185 + j * TILE_H)
            ani.start
            anims << ani
            interface.mars_map[i][j] = nil
          end
        end
      end
    end

    # process animations
    anims.each do |ani|
      ani.render(screen)
      anims.delete(ani) if not ani.running
    end
  end
  
  screen.flip
  frame_counter += 1
  SDL.delay(200 / TARGET_FPS)
end
