STR_COSTS_CREDITS = [300, 20, 150, 350, 25, 10, 50, 230, 1000]
STR_COSTS_RESEARCH = [0, 0, 20, 55, 0, 0, 5, 15, 100]
STR_COSTS_PEOPLE = [45, 5, 25, 5, 2, 5, 12, 15, 23]
STR_BONUSES_CREDITS = [-2, 10, 25, 40, -5, -2, -4, -6, -10]
STR_BONUSES_RESEARCH = [4, 0, 2, 0, 0, 0, 0, 0, 0]
STR_BONUSES_PEOPLE = [0, 0, 0, 0, 2, -1, -2, -2, -4]
STR_RANGES = [0, 0, 0, 0, 0, 300, 250, 200, 170]
STR_DAMAGES = [0, 0, 0, 0, 0, 2, 4, 10, 25]
STR_LASERS = [0, 0, 0, 0, 0, 1, 2, 3, 5]
STR_HP = [5000, 1000, 3000, 4000, 1200, 2000, 3500, 4000, 5000]
STR_NAMES = [
  'Research Lab.', 'Factory', 'Adv. Factory', 'Recycling Center', 'Housing',
  'Basic Lsr.', 'Adv. Lsr.', 'Antic Lsr.', 'Huble Lsr.'
]

require 'enemy.rb'

class Structure
  attr_reader :type
  attr_reader :costs
  attr_reader :bonuses
  attr_reader :army
  attr_reader :hp
  attr_writer :hp
  attr_reader :destroyed

  def initialize(type, screen, x, y, w, h)
    @army = true
    @armu = false if type < 5
    @screen = screen
    @type = type
    @costs = [STR_COSTS_CREDITS[type], STR_COSTS_RESEARCH[type], STR_COSTS_PEOPLE[type]]
    @bonuses = [STR_BONUSES_CREDITS[type], STR_BONUSES_RESEARCH[type], STR_BONUSES_PEOPLE[type]]
    @center = AREVector.new(x + w / 2, y + h / 2)
    @range = STR_RANGES[type]
    @hp = STR_HP[type]

    @laser_colors = [0, 0, 0, 0, 0,
    screen.format.mapRGB(0xff, 0x41, 0x1b),
    screen.format.mapRGB(0x23, 0xff, 0xfb),
    screen.format.mapRGB(0x23, 0xff, 0x23),
    screen.format.mapRGB(0xff, 0x29, 0xd1)]

    # load bitmap
    spritemap = SDL::Surface.load('./gfx/structures.bmp')
    @pix = SDL::Surface.new(SDL::HWSURFACE, w, h, screen)

    x = 0
    y = 0
    if (type < 5) then
      x = type * w
    else
      x = (type - 5) * w
      y = h
    end
    SDL.blitSurface(spritemap, x, y, w, h, @pix, 0, 0)
    @pix.setColorKey(SDL::SRCCOLORKEY, @pix[0,0])
    @pix = @pix.displayFormat

    @targets = Array.new()
    @destroyed = false
  end

  def draw(x, y)
    @screen.put(@pix, x, y)
    @screen.lock
    @screen.drawFilledCircleAlpha(x + 4, y + 4, 3, @screen.format.mapRGB(0x17, 0x1b, 0xff), (255 * (@hp.to_f / STR_HP[@type])).round)
    @screen.unlock

    @destroyed = true if @hp <= 0
  end

  def fire(aliens, screen)
    (STR_LASERS[@type] - @targets.size).times do |i|
      # select target in sight
      inrange = Array.new
      aliens.each do |alien|
        dist = Math.sqrt((alien.position - @center).sumsq)
        inrange << alien if (dist < @range)
      end
      t = inrange[rand(inrange.size)]
      @targets << t if not inrange.empty? and not @targets.include?(t)
    end

    @targets.each do |target|
      if not target.nil?
        # fire!
        tx = target.position.x + 8
        ty = target.position.y + 8
        screen.lock
        screen.drawLineAlpha(@center.x, @center.y, tx, ty, @laser_colors[type], 200)
        screen.drawLineAlpha(@center.x + 2, @center.y + 2, tx, ty, @laser_colors[type], 200)
        screen.drawLineAlpha(@center.x - 2, @center.y - 2, tx, ty, @laser_colors[type], 200)
        screen.unlock
        target.hp -= STR_DAMAGES[type]
        # is target still valid?
        dist = Math.sqrt((target.position - @center).sumsq)
        @targets.delete(target) if target.out or dist > @range
      end
    end
  end
end